import os
import time
import psycopg2
import boto3
import json
import logging
from datetime import datetime

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Log environment variable retrieval
aws_region = os.environ.get('AWS_REGION')
rds_hostname = os.environ.get('RDS_HOSTNAME')
rds_password = os.environ.get('RDS_PASSWORD')
rds_username = os.environ.get('RDS_USERNAME')
rds_port = os.environ.get('RDS_PORT')
rds_db_name = os.environ.get('RDS_DB_NAME')
s3_bucket_name = os.environ.get('S3_BUCKET_NAME')

logger.info('Retrieved environment variables:')
logger.info(f'AWS_REGION: {aws_region}')
logger.info(f'RDS_HOSTNAME: {rds_hostname}')
logger.info(f'RDS_PASSWORD: ********')  # Mask password
logger.info(f'RDS_USERNAME: {rds_username}')
logger.info(f'RDS_PORT: {rds_port}')
logger.info(f'RDS_DB_NAME: {rds_db_name}')
logger.info(f'S3_BUCKET_NAME: {s3_bucket_name}')

# RDS connection details
bucket_name = s3_bucket_name
folder_name = f'hbfw-export/{time.strftime("%Y-%m-%d_%H-%M-%S")}'

try:
    logger.info('Connecting to the RDS database...')
    # Create a PostgreSQL connection using psycopg2
    conn = psycopg2.connect(
        host=rds_hostname,
        port=rds_port,
        database=rds_db_name,
        user=rds_username,
        password=rds_password,
    )

    logger.info('Connected to the RDS database.')

    # Create a cursor to execute SQL commands
    cursor = conn.cursor()

    # SQL command to execute
    sql_command = f"""
        SELECT column_name
        FROM information_schema.columns
        WHERE table_name = 'hbfw_rule_hits_summary'
        ORDER BY ordinal_position
    """

    # Execute the SQL command to fetch the column names
    cursor.execute(sql_command)
    column_names = [row[0] for row in cursor.fetchall()]

    # SQL command to execute
    sql_command_data = f"""
        SELECT *
        FROM hbfw_rule_hits_summary
    """

    # Execute the SQL command to fetch the data
    cursor.execute(sql_command_data)
    results = cursor.fetchall()

    # Convert the data to a list of dictionaries (JSON-like records)
    data_json_list = []
    for row in results:
        record = {}
        for i in range(len(column_names)):
            if isinstance(row[i], datetime):
                record[column_names[i]] = row[i].strftime('%Y-%m-%d %H:%M:%S')
            else:
                record[column_names[i]] = row[i]
        data_json_list.append(record)

    # Write the data to a JSON file in S3
    s3_client = boto3.client('s3')
    s3_client.put_object(
        Bucket=bucket_name,
        Key=f'{folder_name}/data.json',
        Body=json.dumps(data_json_list),
        ContentType='application/json'
    )
    logger.info('Data stored successfully in S3.')

except Exception as e:
    logger.error('Error connecting to the RDS database or storing data in S3: %s', e)

finally:
    # Close the cursor and connection
    if cursor is not None:
        cursor.close()

    if conn is not None:
        conn.close()
